'''actstream_autoactor utilities.'''

import contextlib
import threading

ACTOR_STORAGE = threading.local()
ACTOR_STORAGE.actor = None


def get_actor():
    '''Return the currently set actor.'''
    return ACTOR_STORAGE.actor


def set_actor(actor):
    '''Set the current actor.'''
    ACTOR_STORAGE.actor = actor


@contextlib.contextmanager
def actor_context(actor):
    '''Set the current actor as a context manager.'''
    old_actor = get_actor()
    set_actor(actor)
    yield
    set_actor(old_actor)



"""Tests for actstream_autoactor"""

from django.contrib.auth.models import User
from django.test import TestCase

from mock import Mock

from .middleware import SetActorMiddleware
from .utils import actor_context, get_actor


class CurrentActorTestCase(TestCase):
    """Testing the current actor."""

    def setUp(self):
        self.user1 = User.objects.create(username='user1')
        self.user2 = User.objects.create(username='user2')

    def test_context_manager(self):
        '''Test the context manager.'''

        self.assertEqual(get_actor(), None)

        with actor_context(self.user1):
            self.assertEqual(get_actor(), self.user1)

        self.assertEqual(get_actor(), None)

    def test_middleware(self):
        '''Test the middleware.'''

        context_manager = SetActorMiddleware()
        request = Mock()
        request.user = self.user2
        response = Mock()

        self.assertEqual(get_actor(), None)
        self.assertEqual(context_manager.process_request(request), None)
        self.assertEqual(get_actor(), self.user2)
        self.assertEqual(
            context_manager.process_response(request, response),
            response,
        )
        self.assertEqual(get_actor(), None)



'''actstream_autoactor middleware.'''

from .utils import get_actor, set_actor


class SetActorMiddleware(object):
    '''
    Set the current actor to be the logged in user.
    '''

    def process_request(self, request):
        '''
        Set the current actor.
        '''
        request.actstream_actor_old_actor = get_actor()
        if not request.user.is_anonymous():
            set_actor(request.user)

    def process_response(self, request, response):
        '''
        Unset the current actor.
        '''
        set_actor(request.actstream_actor_old_actor)
        return response



'''Automatic configuration for actstream_autoactor.'''

from django_autoconfig.autoconfig import OrderingRelationship

SETTINGS = {
    'INSTALLED_APPS': [
        'django.contrib.auth',
    ],

    'MIDDLEWARE_CLASSES': [
        'actstream_autoactor.middleware.SetActorMiddleware',
    ],
}

RELATIONSHIPS = [
    OrderingRelationship(
        'MIDDLEWARE_CLASSES',
        'actstream_autoactor.middleware.SetActorMiddleware',
        after=[
            'django.contrib.auth.middleware.AuthenticationMiddleware',
        ],
    ),
]



"""Setup file for django-activity-stream-autoactor."""
from setuptools import setup, find_packages

from actstream_autoactor.version import __VERSION__

setup(
    name='django-activity-stream-autoactor',
    version=__VERSION__,
    packages=find_packages(),
    author='Mike Bryant',
    author_email='mike@mikebryant.me.uk',
    description='Automatic provision of actors for django-activity-stream',
    long_description=open('README.rst').read(),
    url='https://github.com/mikebryant/django-activity-stream-autoactor/',
    install_requires=[
        'django',
        'django-autoconfig',
    ],
    tests_require=[
        'django-setuptest',
        'mock',
    ],
    test_suite='setuptest.setuptest.SetupTestSuite',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
